#include "RooRealVar.h"
#include "RooDataSet.h"
#include "RooDataHist.h"
#include "RooPlot.h"
#include "RooGenericPdf.h"

#include "TCanvas.h"
#include "TMath.h"
#include "TFile.h"
#include "TNtupleD.h"
#include "TH1D.h"
#include "TROOT.h"

#include <sstream>

using namespace RooFit;
using namespace std;

TNtupleD *RooFitNc(const char* fname)
{
  auto   fp   = new TFile(fname);
  auto   hNcp = static_cast<TNtupleD *>(gROOT->FindObject("hNc"));

#if 0
  int    nmax =  2000.;
  double xmin =   200.;
#else
  int    nmax =  3000.;
  double xmin =   200.;
#endif
  double xmax =   nmax;
  auto   xp   = new RooRealVar("nc", "nc", xmin, xmax);
  auto  &x    = *xp;
#if 1
  auto   datp = new RooDataSet("data","dataset with x",hNcp,x);
#else
  stringstream cmdstr;
  cmdstr << "nc>>h(" << nmax << 0. << "," << nmax << ")" << ends;
  hNcp->Draw(cmdstr.str().data());
  auto   hp = static_cast<TH1D *>(gROOT->FindObject("h"));
  auto   datp = new RooDataHist("data","dataset with x",x,hp);
#endif
  auto  &data = *datp;
  auto   xfp  = x.frame();
  
#if 1
  data.plotOn(xfp);
  xfp->Draw();
#endif
  
  auto  p200p  = new RooRealVar("p200" ,"p200" ,0.1,0.,1.);
  auto  alphap = new RooRealVar("alpha","alpha",2.0,1.,3.);
  auto  betap  = new RooRealVar("beta" ,"beta" ,0.0,0.,1.);
  auto &p200  = *p200p;
  auto &alpha = *alphap;
  auto &beta  = *betap;
  auto  rpdfp = new RooGenericPdf("rpdf","p200*exp(-beta*x)/pow(x,alpha)",RooArgSet(x,p200,alpha,beta));
  rpdfp->fitTo(data,Range(xmin,xmax));
  rpdfp->plotOn(xfp);
  p200.Print();
  alpha.Print();
  xfp->Draw();
#if 0
  auto   x2p   = new RooRealVar("x2", "x2", 0., xmax);
  auto  &x2    = *x2p;
  auto   dat2p = rpdfp->generate(x2,1000);
  auto  &data2 = *dat2p;
  auto   x2fp  = x2.frame();
  data2.plotOn(x2fp);
  x2fp->Draw();
#endif

  return hNcp;
}
